@@ -6,7 +6,7 @@ namespace IBus {
 
 ConfigService::ConfigService (const QDBusConnection &conn) : m_conn(conn)
 {
-    m_config = new IBusConfigAdaptor(this);
+    m_config = new IBusConfigAdaptor (this);
     if ( !m_config ) {
         qDebug () << "ConfigService::ConfigService, new IBusConfigAdaptor error!";
     }
@@ -20,16 +20,22 @@ ConfigService::~ConfigService ()
     }
 }
 
-const QDBusVariant &
+const QDBusVariant 
 ConfigService::GetValue (const QString &section, const QString &name)
 {
-    return getValue(section, name);
+    return getValue (section, name);
 }
 
 void
 ConfigService::SetValue (const QString &section, const QString &name, const QDBusVariant &value)
 {
-    setValue(section, name, value);
+    setValue (section, name, value);
+}
+
+void
+ConfigService::Unset (const QString &section, const QString &name)
+{
+    unset (section, name);
 }
 
 void
